@@ -1,18 +1,19 @@
-## This file is part of Scapy
-## See http://www.secdev.org/projects/scapy for more informations
-## Copyright (C) Philippe Biondi <phil@secdev.org>
-## This program is published under a GPLv2 license
+# This file is part of Scapy
+# See http://www.secdev.org/projects/scapy for more information
+# Copyright (C) Philippe Biondi <phil@secdev.org>
+# This program is published under a GPLv2 license
 
 """
 Color themes for the interactive console.
 """
 
 ##################
-## Color themes ##
+#  Color themes  #
 ##################
 
+
 class ColorTable:
-    colors = { # Format: (ansi, pygments)
+    colors = {  # Format: (ansi, pygments)
         "normal": ("\033[0m", "noinherit"),
         "black": ("\033[30m", "#ansiblack"),
         "red": ("\033[31m", "#ansired"),
@@ -27,22 +28,24 @@ class ColorTable:
         "uline": ("\033[4m", "underline"),
         "blink": ("\033[5m", ""),
         "invert": ("\033[7m", ""),
-        }
+    }
 
     def __repr__(self):
         return "<ColorTable>"
 
     def __getattr__(self, attr):
         return self.colors.get(attr, [""])[0]
-    
-    def ansi_to_pygments(self, x): # Transform ansi encoded text to Pygments text
+
+    def ansi_to_pygments(self, x):  # Transform ansi encoded text to Pygments text  # noqa: E501
         inv_map = {v[0]: v[1] for k, v in self.colors.items()}
         for k, v in inv_map.items():
-            x = x.replace(k, " "+v)
+            x = x.replace(k, " " + v)
         return x.strip()
-        
+
+
 Color = ColorTable()
 
+
 def create_styler(fmt=None, before="", after="", fmt2="%s"):
     def do_style(val, fmt=fmt, before=before, after=after, fmt2=fmt2):
         if fmt is None:
@@ -50,20 +53,23 @@ def create_styler(fmt=None, before="", after="", fmt2="%s"):
                 val = str(val)
         else:
             val = fmt % val
-        return fmt2 % (before+val+after)
+        return fmt2 % (before + val + after)
     return do_style
 
+
 class ColorTheme:
     def __repr__(self):
         return "<%s>" % self.__class__.__name__
+
     def __reduce__(self):
         return (self.__class__, (), ())
+
     def __getattr__(self, attr):
         if attr in ["__getstate__", "__setstate__", "__getinitargs__",
                     "__reduce_ex__"]:
             raise AttributeError()
         return create_styler()
-        
+
 
 class NoTheme(ColorTheme):
     pass
@@ -73,7 +79,7 @@ class AnsiColorTheme(ColorTheme):
     def __getattr__(self, attr):
         if attr.startswith("__"):
             raise AttributeError(attr)
-        s = "style_%s" % attr 
+        s = "style_%s" % attr
         if s in self.__class__.__dict__:
             before = getattr(self, s)
             after = self.style_normal
@@ -81,8 +87,7 @@ class AnsiColorTheme(ColorTheme):
             before = after = ""
 
         return create_styler(before=before, after=after)
-        
-        
+
     style_normal = ""
     style_prompt = ""
     style_punct = ""
@@ -107,103 +112,106 @@ class AnsiColorTheme(ColorTheme):
     style_right = ""
     style_logo = ""
 
+
 class BlackAndWhite(AnsiColorTheme):
     pass
 
+
 class DefaultTheme(AnsiColorTheme):
     style_normal = Color.normal
-    style_prompt = Color.blue+Color.bold
+    style_prompt = Color.blue + Color.bold
     style_punct = Color.normal
-    style_id = Color.blue+Color.bold
+    style_id = Color.blue + Color.bold
     style_not_printable = Color.grey
-    style_layer_name = Color.red+Color.bold
+    style_layer_name = Color.red + Color.bold
     style_field_name = Color.blue
     style_field_value = Color.purple
-    style_emph_field_name = Color.blue+Color.uline+Color.bold
-    style_emph_field_value = Color.purple+Color.uline+Color.bold
-    style_packetlist_name = Color.red+Color.bold
+    style_emph_field_name = Color.blue + Color.uline + Color.bold
+    style_emph_field_value = Color.purple + Color.uline + Color.bold
+    style_packetlist_name = Color.red + Color.bold
     style_packetlist_proto = Color.blue
     style_packetlist_value = Color.purple
-    style_fail = Color.red+Color.bold
-    style_success = Color.blue+Color.bold
-    style_even = Color.black+Color.bold
+    style_fail = Color.red + Color.bold
+    style_success = Color.blue + Color.bold
+    style_even = Color.black + Color.bold
     style_odd = Color.black
     style_opening = Color.yellow
     style_active = Color.black
     style_closed = Color.grey
-    style_left = Color.blue+Color.invert
-    style_right = Color.red+Color.invert
-    style_logo = Color.green+Color.bold
-    
+    style_left = Color.blue + Color.invert
+    style_right = Color.red + Color.invert
+    style_logo = Color.green + Color.bold
+
+
 class BrightTheme(AnsiColorTheme):
     style_normal = Color.normal
     style_punct = Color.normal
-    style_id = Color.yellow+Color.bold
-    style_layer_name = Color.red+Color.bold
-    style_field_name = Color.yellow+Color.bold
-    style_field_value = Color.purple+Color.bold
-    style_emph_field_name = Color.yellow+Color.bold
-    style_emph_field_value = Color.green+Color.bold
-    style_packetlist_name = Color.red+Color.bold
-    style_packetlist_proto = Color.yellow+Color.bold
-    style_packetlist_value = Color.purple+Color.bold
-    style_fail = Color.red+Color.bold
-    style_success = Color.blue+Color.bold
-    style_even = Color.black+Color.bold
+    style_id = Color.yellow + Color.bold
+    style_layer_name = Color.red + Color.bold
+    style_field_name = Color.yellow + Color.bold
+    style_field_value = Color.purple + Color.bold
+    style_emph_field_name = Color.yellow + Color.bold
+    style_emph_field_value = Color.green + Color.bold
+    style_packetlist_name = Color.red + Color.bold
+    style_packetlist_proto = Color.yellow + Color.bold
+    style_packetlist_value = Color.purple + Color.bold
+    style_fail = Color.red + Color.bold
+    style_success = Color.blue + Color.bold
+    style_even = Color.black + Color.bold
     style_odd = Color.black
-    style_left = Color.cyan+Color.invert
-    style_right = Color.purple+Color.invert
-    style_logo = Color.green+Color.bold
+    style_left = Color.cyan + Color.invert
+    style_right = Color.purple + Color.invert
+    style_logo = Color.green + Color.bold
 
 
 class RastaTheme(AnsiColorTheme):
-    style_normal = Color.normal+Color.green+Color.bold
-    style_prompt = Color.yellow+Color.bold
+    style_normal = Color.normal + Color.green + Color.bold
+    style_prompt = Color.yellow + Color.bold
     style_punct = Color.red
-    style_id = Color.green+Color.bold
+    style_id = Color.green + Color.bold
     style_not_printable = Color.green
-    style_layer_name = Color.red+Color.bold
-    style_field_name = Color.yellow+Color.bold
-    style_field_value = Color.green+Color.bold
+    style_layer_name = Color.red + Color.bold
+    style_field_name = Color.yellow + Color.bold
+    style_field_value = Color.green + Color.bold
     style_emph_field_name = Color.green
     style_emph_field_value = Color.green
-    style_packetlist_name = Color.red+Color.bold
-    style_packetlist_proto = Color.yellow+Color.bold
-    style_packetlist_value = Color.green+Color.bold
+    style_packetlist_name = Color.red + Color.bold
+    style_packetlist_proto = Color.yellow + Color.bold
+    style_packetlist_value = Color.green + Color.bold
     style_fail = Color.red
-    style_success = Color.red+Color.bold
+    style_success = Color.red + Color.bold
     style_even = Color.yellow
     style_odd = Color.green
-    style_left = Color.yellow+Color.invert
-    style_right = Color.red+Color.invert
-    style_logo = Color.green+Color.bold
+    style_left = Color.yellow + Color.invert
+    style_right = Color.red + Color.invert
+    style_logo = Color.green + Color.bold
 
 
 class ColorOnBlackTheme(AnsiColorTheme):
     """Color theme for black backgrounds"""
     style_normal = Color.normal
-    style_prompt = Color.green+Color.bold
+    style_prompt = Color.green + Color.bold
     style_punct = Color.normal
     style_id = Color.green
-    style_not_printable = Color.black+Color.bold
-    style_layer_name = Color.yellow+Color.bold
+    style_not_printable = Color.black + Color.bold
+    style_layer_name = Color.yellow + Color.bold
     style_field_name = Color.cyan
-    style_field_value = Color.purple+Color.bold
-    style_emph_field_name = Color.cyan+Color.bold
-    style_emph_field_value = Color.red+Color.bold
-    style_packetlist_name = Color.black+Color.bold
-    style_packetlist_proto = Color.yellow+Color.bold
-    style_packetlist_value = Color.purple+Color.bold
-    style_fail = Color.red+Color.bold
+    style_field_value = Color.purple + Color.bold
+    style_emph_field_name = Color.cyan + Color.bold
+    style_emph_field_value = Color.red + Color.bold
+    style_packetlist_name = Color.black + Color.bold
+    style_packetlist_proto = Color.yellow + Color.bold
+    style_packetlist_value = Color.purple + Color.bold
+    style_fail = Color.red + Color.bold
     style_success = Color.green
-    style_even = Color.black+Color.bold
+    style_even = Color.black + Color.bold
     style_odd = Color.grey
     style_opening = Color.yellow
-    style_active = Color.grey+Color.bold
-    style_closed = Color.black+Color.bold
-    style_left = Color.cyan+Color.bold
-    style_right = Color.red+Color.bold
-    style_logo = Color.green+Color.bold
+    style_active = Color.grey + Color.bold
+    style_closed = Color.black + Color.bold
+    style_left = Color.cyan + Color.bold
+    style_right = Color.red + Color.bold
+    style_logo = Color.green + Color.bold
 
 
 class FormatTheme(ColorTheme):
@@ -211,7 +219,8 @@ class FormatTheme(ColorTheme):
         if attr.startswith("__"):
             raise AttributeError(attr)
         colfmt = self.__class__.__dict__.get("style_%s" % attr, "%s")
-        return create_styler(fmt2 = colfmt)       
+        return create_styler(fmt2=colfmt)
+
 
 class LatexTheme(FormatTheme):
     style_prompt = r"\textcolor{blue}{%s}"
@@ -219,8 +228,8 @@ class LatexTheme(FormatTheme):
     style_layer_name = r"\textcolor{red}{\bf %s}"
     style_field_name = r"\textcolor{blue}{%s}"
     style_field_value = r"\textcolor{purple}{%s}"
-    style_emph_field_name = r"\textcolor{blue}{\underline{%s}}" #ul
-    style_emph_field_value = r"\textcolor{purple}{\underline{%s}}" #ul
+    style_emph_field_name = r"\textcolor{blue}{\underline{%s}}"  # ul
+    style_emph_field_value = r"\textcolor{purple}{\underline{%s}}"  # ul
     style_packetlist_name = r"\textcolor{red}{\bf %s}"
     style_packetlist_proto = r"\textcolor{blue}{%s}"
     style_packetlist_value = r"\textcolor{purple}{%s}"
@@ -232,14 +241,15 @@ class LatexTheme(FormatTheme):
 #    style_odd = ""
     style_logo = r"\textcolor{green}{\bf %s}"
 
+
 class LatexTheme2(FormatTheme):
     style_prompt = r"@`@textcolor@[@blue@]@@[@%s@]@"
     style_not_printable = r"@`@textcolor@[@gray@]@@[@%s@]@"
     style_layer_name = r"@`@textcolor@[@red@]@@[@@`@bfseries@[@@]@%s@]@"
     style_field_name = r"@`@textcolor@[@blue@]@@[@%s@]@"
     style_field_value = r"@`@textcolor@[@purple@]@@[@%s@]@"
-    style_emph_field_name = r"@`@textcolor@[@blue@]@@[@@`@underline@[@%s@]@@]@" 
-    style_emph_field_value = r"@`@textcolor@[@purple@]@@[@@`@underline@[@%s@]@@]@" 
+    style_emph_field_name = r"@`@textcolor@[@blue@]@@[@@`@underline@[@%s@]@@]@"
+    style_emph_field_value = r"@`@textcolor@[@purple@]@@[@@`@underline@[@%s@]@@]@"  # noqa: E501
     style_packetlist_name = r"@`@textcolor@[@red@]@@[@@`@bfseries@[@@]@%s@]@"
     style_packetlist_proto = r"@`@textcolor@[@blue@]@@[@%s@]@"
     style_packetlist_value = r"@`@textcolor@[@purple@]@@[@%s@]@"
@@ -251,6 +261,7 @@ class LatexTheme2(FormatTheme):
     style_right = r"@`@textcolor@[@red@]@@[@%s@]@"
     style_logo = r"@`@textcolor@[@green@]@@[@@`@bfseries@[@@]@%s@]@"
 
+
 class HTMLTheme(FormatTheme):
     style_prompt = "<span class=prompt>%s</span>"
     style_not_printable = "<span class=not_printable>%s</span>"
@@ -269,6 +280,7 @@ class HTMLTheme(FormatTheme):
     style_left = "<span class=left>%s</span>"
     style_right = "<span class=right>%s</span>"
 
+
 class HTMLTheme2(HTMLTheme):
     style_prompt = "#[#span class=prompt#]#%s#[#/span#]#"
     style_not_printable = "#[#span class=not_printable#]#%s#[#/span#]#"
@@ -293,23 +305,24 @@ def apply_ipython_style(shell):
     the conf.color_theme scapy theme."""
     try:
         from IPython.terminal.prompts import Prompts, Token
-    except:
+    except Exception:
         from scapy.error import log_loading
         log_loading.warning(
             "IPython too old. Shell color won't be handled."
-            )
+        )
         return
     from scapy.config import conf
     if isinstance(conf.prompt, Prompts):
-        shell.prompts_class = conf.prompt # Set custom prompt style
+        shell.prompts_class = conf.prompt  # Set custom prompt style
     else:
         class ClassicPrompt(Prompts):
             def in_prompt_tokens(self, cli=None):
-               return [(Token.Prompt, str(conf.prompt)),]
+                return [(Token.Prompt, str(conf.prompt)), ]
+
             def out_prompt_tokens(self):
-               return [(Token.OutPrompt, ''),]
-        shell.prompts_class=ClassicPrompt # Apply classic prompt style
-    shell.highlighting_style_overrides = { # Register and apply scapy color style
+                return [(Token.OutPrompt, ''), ]
+        shell.prompts_class = ClassicPrompt  # Apply classic prompt style
+    shell.highlighting_style_overrides = {  # Register and apply scapy color style  # noqa: E501
         Token.Prompt: Color.ansi_to_pygments(conf.color_theme.style_prompt),
     }
     try:
